<?php

declare(strict_types=1);

namespace Drupal\graphql_compose_preview\KeyValueStore;

use Drupal\Component\Utility\Crypt;
use Drupal\Core\KeyValueStore\DatabaseStorageExpirable;

/**
 * Utility methods to interact with the tokened private temp store.
 */
class DatabaseStorageExpirableToken extends DatabaseStorageExpirable {

  /**
   * {@inheritdoc}
   *
   * Add preview_token.
   */
  protected function doSetWithExpire($key, $value, $expire) {
    $this->connection->merge($this->table)
      ->keys([
        'name' => $key,
        'collection' => $this->collection,
      ])
      ->fields([
        'value' => $this->serializer->encode($value),
        'expire' => REQUEST_TIME + $expire,
        'preview_token' => Crypt::randomBytesBase64(64),
      ])
      ->execute();
  }

  /**
   * Load by token.
   *
   * @param string $token
   *   The token to load from the private temp store.
   *
   * @return string|bool
   *   The value.
   */
  public function getKeyByToken($token) {
    try {
      return $this->connection->query(
        'SELECT [name] FROM {' . $this->connection->escapeTable($this->table) . '} WHERE preview_token = :token',
        [
          ':token' => $token,
        ])->fetchField();
    }
    catch (\Exception $e) {
      $this->catchException($e);
    }
    return FALSE;
  }

  /**
   * Load by token.
   *
   * @param string $name
   *   The name to load from the private temp store.
   *
   * @return string|bool
   *   The value.
   */
  public function getTokenByKey($name) {
    try {
      return $this->connection->query(
        'SELECT [preview_token] FROM {' . $this->connection->escapeTable($this->table) . '} WHERE name = :name',
        [
          ':name' => $name,
        ])->fetchField();
    }
    catch (\Exception $e) {
      $this->catchException($e);
    }
    return FALSE;
  }

}
